/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Tests the functionality and error cases of <c>BreadCrumbNode</c> structure. Since this is an immutable class,
    /// the property getters are tested together with the constructor.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class BreadCrumbNodeTestCase
    {
        /// <summary>
        /// Tests accuracy of <c>BreadCrumbNode(string, string)</c> when the given title is <c>null</c>. The title
        /// should be set as <c>null</c>.
        /// </summary>
        [Test]
        public void TestBreadCrumbNodeStringStringTitleNullAccuracy()
        {
            BreadCrumbNode node = new BreadCrumbNode(null, "url");

            // Verify
            Assert.IsNull(node.Title, "The title should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbNode(string, string)</c> when the given title is empty string. The title
        /// should be set as <c>null</c>.
        /// </summary>
        [Test]
        public void TestBreadCrumbNodeStringStringTitleEmptyAccuracy()
        {
            BreadCrumbNode node = new BreadCrumbNode("   ", "url");

            // Verify
            Assert.IsNull(node.Title, "The title should be set as null.");
        }

        /// <summary>
        /// Tests <c>BreadCrumbNode(string, string)</c> when the given URL is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestBreadCrumbNodeStringStringURLNull()
        {
            new BreadCrumbNode("title", null);
        }

        /// <summary>
        /// Tests <c>BreadCrumbNode(string, string)</c> when the given URL is empty string. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestBreadCrumbNodeStringStringURLEmpty()
        {
            new BreadCrumbNode("title", "  ");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbNode(string, string)</c>. The given title and the URL should be able to be
        /// retrieved by the property getters. This is also a test case for the property getters. The title should be
        /// 'title', and the URL should be 'url'.
        /// </summary>
        [Test]
        public void TestBreadCrumbNodeStringStringAccuracy()
        {
            BreadCrumbNode node = new BreadCrumbNode("title", "url");

            // Verify
            Assert.AreEqual("title", node.Title, "The title should be 'title' as given.");
            Assert.AreEqual("url", node.Url, "The URL should be 'url' as given.");
        }
    }
}

