/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;
using NUnit.Framework;
using NUnit.Extensions.Asp;
using NUnit.Extensions.Asp.AspTester;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.NodeDiscovery
{
    /// <summary>
    /// Tests the functionality and error cases of <c>HttpRequestNodeDiscovery</c> class. Some test cases are in the
    /// ASP.NET page, and accessed via NUnitAsp.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class HttpRequestNodeDiscoveryTestCase : WebFormTestCase
    {
        /// <summary>
        /// Represents the <c>HttpRequestNodeDiscovery</c> instance used in tests.
        /// </summary>
        private static readonly INodeDiscovery nodeDiscovery = new HttpRequestNodeDiscovery();

        /// <summary>
        /// Tests <c>GetCurrentNode(string, Page)</c> when the given page is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestGetCurrentNodeStringPagePageNull()
        {
            nodeDiscovery.GetCurrentNode("title", null);
        }

        /// <summary>
        /// Tests <c>GetCurrentNode(string, Page)</c> when the given page does not have HTTP request. Expect
        /// BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetCurrentNodeStringPageNoRequest()
        {
            nodeDiscovery.GetCurrentNode("title", new Page());
        }

        /// <summary>
        /// Tests accuracy of <c>GetCurrentNode(string, Page)</c> when the HTTP request of the given page does not have
        /// a query string. The returned bread crumb node should have the given title and the URL of the HTTP request.
        /// This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetCurrentNodeStringPageNoQueryStringAccuracy()
        {
            Browser.GetPage("http://localhost/test/HttpRequestNodeDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetCurrentNodeStringPageNoQueryStringAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetCurrentNode(string, Page)</c> when the HTTP request of the given page has a query
        /// string. The returned bread crumb node should have the given title and the URL of the HTTP request. The query
        /// string should be sorted according to the property names. This is verified by the ASP.NET test page via
        /// NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetCurrentNodeStringPageQueryStringAccuracy()
        {
            Browser.GetPage("http://localhost/test/HttpRequestNodeDiscoveryTest.aspx?query=abc&print=exe&execute=cmd");

            ButtonTester button = new ButtonTester("GetCurrentNodeStringPageQueryStringAccuracy", CurrentWebForm);

            button.Click();
        }
    }
}

