<%@ Page Language="C#" Debug="true" %>

<%@ Import Namespace="TopCoder.Web.UI.WebControl.BreadCrumb" %>
<%@ Import Namespace="TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery" %>
<%@ Import Namespace="TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher" %>
<%@ Import Namespace="TopCoder.Util.ConfigurationManager" %>
<%@ Import Namespace="NUnit.Framework" %>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN">

<script runat="server">
    /// <summary>
    /// Represents the <c>DynamicPathDiscovery</c> instance used in tests.
    /// </summary>
    private IPathDiscovery pathDiscovery;

    /// <summary>
    /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the reversed site map is invalid. Expect BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageSiteMapInvalid(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/Test.xml"));

        BreadCrumbNode node = new BreadCrumbNode("title", "/normal.aspx");

        // Set the invalid site map
        IDictionary map = new Hashtable();

        Page.Session[DynamicPathDiscovery.CURRENT_SITEMAP] = map;
        Page.Session[DynamicPathDiscovery.CURRENT_REVERSE_SITEMAP] = map;

        map.Add(new RegexNodeMatcher("normal", "/normal.aspx", "/normal.*"), new object());

        try
        {
            pathDiscovery.GetPath(node, Page);
            Assert.Fail("The reversed site map is invalid, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when retrieveing the site map fails. Expect BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageGetSiteMapFail(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/InvalidSiteMap.xml"));

        BreadCrumbNode node = new BreadCrumbNode("title", "/normal.aspx");

        try
        {
            pathDiscovery.GetPath(node, Page);
            Assert.Fail("Retrieving site map fails, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when creating the ISiteDiscovery instance fails. Expect
    /// BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageISiteDiscoveryCreationFail(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/CreationError.xml"));

        BreadCrumbNode node = new BreadCrumbNode("title", "/normal.aspx");

        try
        {
            pathDiscovery.GetPath(node, Page);
            Assert.Fail("Creating ISiteDiscovery fails, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the ISiteDiscovery instance cannot be found in the object
    /// factory. Expect BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageISiteDiscoverySourceFail(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/SourceError.xml"));

        BreadCrumbNode node = new BreadCrumbNode("title", "/normal.aspx");

        try
        {
            pathDiscovery.GetPath(node, Page);
            Assert.Fail("The ISiteDiscovery is not available in the object factory, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the ISiteDiscovery instance is not an ISiteDiscovery
    /// instance. Expect BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageISiteDiscoveryWrongType(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/TypeError.xml"));

        BreadCrumbNode node = new BreadCrumbNode("title", "/normal.aspx");

        try
        {
            pathDiscovery.GetPath(node, Page);
            Assert.Fail("The ISiteDiscovery is an ISiteDiscovery instance, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the configuration for the object factory is invalid. Expect
    /// BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageInvalidConfig(object sender, EventArgs e)
    {
        BreadCrumbNode node = new BreadCrumbNode("title", "/normal.aspx");

        try
        {
            pathDiscovery.GetPath(node, Page);
            Assert.Fail("The object factory config is invalid, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Asserts the two given paths are the same. The titles and URLs of the bread crumb nodes must be the same.
    /// </summary>
    /// <param name="expected">The expected path to be compared with.</param>
    /// <param name="actual">The actual path to be compared with.</param>
    /// <param name="message">The error message when the assertion fails.</param>
    /// <exception cref="AssertionException">The actual path is not the same as the expected path.</exception>
    private static void AreEqual(BreadCrumbNode[] expected, BreadCrumbNode[] actual, string message)
    {
        // Verify the counts of redirection
        Assert.AreEqual(expected.Length, actual.Length, message);

        // Verify every item
        for (int i = 0; i < expected.Length; ++i)
        {
            Assert.AreEqual(expected[i].Title, actual[i].Title, message);
            Assert.AreEqual(expected[i].Url, actual[i].Url, message);
        }
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when two matchers matches the given bread crumb node.
    /// The shortest path should be selected. Since the title of the node is <c>null</c>, the title in the matcher is
    /// used.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageTwoMatchedAccuracy(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/Test.xml"));

        BreadCrumbNode node = new BreadCrumbNode(null, "/match3.aspx");

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("main2", "/main2.aspx"),
            new BreadCrumbNode("match2", "/match3.aspx")
        };

        // Verify
        AreEqual(expected, path, "The path should be the shortest one.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when one matched matcher has multiple roots. The
    /// shortest path should be selected. The title of the given bread crumb node should be used.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageMultipleRootsAccuracy(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/Test.xml"));

        BreadCrumbNode node = new BreadCrumbNode("customized", "/dup.aspx");

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("main2", "/main2.aspx"),
            new BreadCrumbNode("customized", "/dup.aspx")
        };

        // Verify
        AreEqual(expected, path, "The path should be the shortest one.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when no matched matcher has been found. There is a
    /// previous path available.  It should return the previous path appending this node.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageNoMatchAccuracy(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/Test.xml"));

        BreadCrumbNode node = new BreadCrumbNode("notfound", "/nomatch.aspx");
        
        // Set the session
        Session[DynamicPathDiscovery.CURRENT_SITEMAP] = new XmlPathDiscovery(Page.MapPath("Valid.xml")).GetSiteMap();
        Session[DynamicPathDiscovery.CURRENT_PATH] = TestHelper.CreateNodePairArray(
            new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx"),
                new BreadCrumbNode("node3", "node3.aspx")
            },
            new INodeMatcher[] {
                new RegexNodeMatcher("main", "/main.aspx", @"/main\.aspx"),
                new RegexNodeMatcher("previous1", "/previous1.aspx", @"/previous1\.aspx"),
                new RegexNodeMatcher("previous2", "/previous2.aspx", @"/previous2\.aspx.*"),
                new RegexNodeMatcher("current", "/current.aspx", @"/current.\.aspx")
            });
        
        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("root", "root.aspx"),
            new BreadCrumbNode("node1", "node1.aspx"),
            new BreadCrumbNode("node2", "node2.aspx"),
            new BreadCrumbNode("node3", "node3.aspx"),
            new BreadCrumbNode("notfound", "/nomatch.aspx"),
        };

        // Get the path
        AreEqual(expected, pathDiscovery.GetPath(node, Page), "The path should be only one node.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when looped matcher matches the node. There is no
    /// previous path available. It should return <c>null</c>, since there is no path to the root.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageLoopAccuracy(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/Test.xml"));

        BreadCrumbNode node = new BreadCrumbNode("nopath", "/nopath.aspx");

        // Get the path
        Assert.IsNull(pathDiscovery.GetPath(node, Page), "There is no path to the root.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when one normal matcher matches the node. It should
    /// return the shortest path.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageNormalAccuracy(object sender, EventArgs e)
    {
        ConfigManager.GetInstance().LoadFile(Page.MapPath("Config/Test.xml"));

        BreadCrumbNode node = new BreadCrumbNode("normal", "/normal.aspx");

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("main", "/main.aspx"),
            new BreadCrumbNode("loop", "/loop.aspx"),
            new BreadCrumbNode("inner", "/inner.aspx"),
            new BreadCrumbNode("match1", "/match1.aspx"),
            new BreadCrumbNode("normal", "/normal.aspx")
        };

        // Verify
        AreEqual(expected, path, "The path should be the shortest one.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the current node is on the previous path. The
    /// previous path should be used directly. The URLs of the previous path and the current node should be
    /// preserved.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageOnPreviousPathAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = new BreadCrumbNode("current2", "/previous2.aspx?whatever");

        // Save the previous path, and the site map
        Session[DynamicPathDiscovery.CURRENT_SITEMAP] = new XmlPathDiscovery(Page.MapPath("Valid.xml")).GetSiteMap();
        Session[DynamicPathDiscovery.CURRENT_PATH] = TestHelper.CreateNodePairArray(
            new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx"),
                new BreadCrumbNode("node3", "node3.aspx")
            },
            new INodeMatcher[] {
                new RegexNodeMatcher("main", "/main.aspx", @"/main\.aspx"),
                new RegexNodeMatcher("previous1", "/previous1.aspx", @"/previous1\.aspx"),
                new RegexNodeMatcher("previous2", "/previous2.aspx", @"/previous2\.aspx.*"),
                new RegexNodeMatcher("current", "/current.aspx", @"/current.\.aspx")
            });

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("root", "root.aspx"),
            new BreadCrumbNode("node1", "node1.aspx"),
            new BreadCrumbNode("current2", "/previous2.aspx?whatever")
        };

        // Verify
        AreEqual(expected, path, "The path should be the cut previous path.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the current node is the child of the last node
    /// in the previous path. The previous path should be used directly appending the current node. The URLs of the
    /// previous path and the current node should be preserved.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageAppendPreviousPathAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = new BreadCrumbNode("current2", "/current2.aspx");

        // Save the previous path, and the site map
        Session[DynamicPathDiscovery.CURRENT_SITEMAP] = new XmlPathDiscovery(Page.MapPath("Valid.xml")).GetSiteMap();
        Session[DynamicPathDiscovery.CURRENT_PATH] = TestHelper.CreateNodePairArray(
            new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx")
            },
            new INodeMatcher[] {
                new RegexNodeMatcher("main", "/main.aspx", @"/main\.aspx"),
                new RegexNodeMatcher("previous1", "/previous1.aspx", @"/previous1\.aspx"),
                new RegexNodeMatcher("previous2", "/previous2.aspx", @"/previous2\.aspx.*")
            });

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("root", "root.aspx"),
            new BreadCrumbNode("node1", "node1.aspx"),
            new BreadCrumbNode("node2", "node2.aspx"),
            new BreadCrumbNode("current2", "/current2.aspx")
        };

        // Verify
        AreEqual(expected, path, "The path should be the previous path appending the current node.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the current node is an unknown node. The
    /// previous path should be used directly appending the current node. The URLs of the previous path and the
    /// current node should be preserved.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePagePreviousPathUnknownAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = new BreadCrumbNode("unknown", "unknown.aspx");

        // Save the previous path, and the site map
        Session[DynamicPathDiscovery.CURRENT_SITEMAP] = new XmlPathDiscovery(Page.MapPath("Valid.xml")).GetSiteMap();
        Session[DynamicPathDiscovery.CURRENT_PATH] = TestHelper.CreateNodePairArray(
            new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx")
            },
            new INodeMatcher[] {
                new RegexNodeMatcher("main", "/main.aspx", @"/main\.aspx"),
                new RegexNodeMatcher("previous1", "/previous1.aspx", @"/previous1\.aspx"),
                new RegexNodeMatcher("previous2", "/previous2.aspx", @"/previous2\.aspx.*")
            });

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("root", "root.aspx"),
            new BreadCrumbNode("node1", "node1.aspx"),
            new BreadCrumbNode("node2", "node2.aspx"),
            new BreadCrumbNode("unknown", "unknown.aspx")
        };

        // Verify
        AreEqual(expected, path, "The path should be the previous path appending the current node.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the shortest path shares no node with the previous
    /// path while there is a longer path which shares some nodes with the previous path. The shortest no common path
    /// should be chosen.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageShortestNoCommonAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = new BreadCrumbNode("shortest2", "/shortest2.aspx");

        // Save the previous path, and the site map
        Session[DynamicPathDiscovery.CURRENT_SITEMAP] = new XmlPathDiscovery(Page.MapPath("Valid.xml")).GetSiteMap();
        Session[DynamicPathDiscovery.CURRENT_PATH] = TestHelper.CreateNodePairArray(
            new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx"),
                new BreadCrumbNode("node3", "node3.aspx")
            },
            new INodeMatcher[] {
                new RegexNodeMatcher("main", "/main.aspx", @"/main\.aspx"),
                new RegexNodeMatcher("previous1", "/previous1.aspx", @"/previous1\.aspx"),
                new RegexNodeMatcher("previous2", "/previous2.aspx", @"/previous2\.aspx.*"),
                new RegexNodeMatcher("current", "/current.aspx", @"/current.\.aspx")
            });

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("main2", "/main2.aspx"),
            new BreadCrumbNode("shortest2", "/shortest2.aspx"),
        };

        // Verify
        AreEqual(expected, path, "The path should be the shortest no common path.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the shortest path shares less node with the
    /// previous path while there is a longer path which shares more nodes with the previous path. The shortest less
    /// common path should be chosen.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageShortestLessCommonAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = new BreadCrumbNode("shortest", "/shortest.aspx");

        // Save the previous path, and the site map
        Session[DynamicPathDiscovery.CURRENT_SITEMAP] = new XmlPathDiscovery(Page.MapPath("Valid.xml")).GetSiteMap();
        Session[DynamicPathDiscovery.CURRENT_PATH] = TestHelper.CreateNodePairArray(
            new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx"),
                new BreadCrumbNode("node3", "node3.aspx")
            },
            new INodeMatcher[] {
                new RegexNodeMatcher("main", "/main.aspx", @"/main\.aspx"),
                new RegexNodeMatcher("previous1", "/previous1.aspx", @"/previous1\.aspx"),
                new RegexNodeMatcher("previous2", "/previous2.aspx", @"/previous2\.aspx.*"),
                new RegexNodeMatcher("current", "/current.aspx", @"/current.\.aspx")
            });

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("root", "root.aspx"),
            new BreadCrumbNode("node1", "node1.aspx"),
            new BreadCrumbNode("shortest", "/shortest.aspx"),
        };

        // Verify
        AreEqual(expected, path, "The path should be the shortest less common path.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the two shortest paths exists. The path which
    /// shares more common nodes with the previous path should be chosen.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetPathBreadCrumbNodePageMoreCommonAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = new BreadCrumbNode("current", "/current1.aspx");

        // Save the previous path, and the site map
        Session[DynamicPathDiscovery.CURRENT_SITEMAP] = new XmlPathDiscovery(Page.MapPath("Valid.xml")).GetSiteMap();
        Session[DynamicPathDiscovery.CURRENT_PATH] = TestHelper.CreateNodePairArray(
            new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
            },
            new INodeMatcher[] {
                new RegexNodeMatcher("main", "/main.aspx", @"/main\.aspx"),
                new RegexNodeMatcher("previous1", "/previous1.aspx", @"/previous1\.aspx"),
            });

        // Get the path
        BreadCrumbNode[] path = pathDiscovery.GetPath(node, Page);

        // Build the expected array
        BreadCrumbNode[] expected = new BreadCrumbNode[] {
            new BreadCrumbNode("root", "root.aspx"),
            new BreadCrumbNode("node1", "node1.aspx"),
            new BreadCrumbNode("previous2", "/previous2.aspx"),
            new BreadCrumbNode("current", "/current1.aspx"),
        };

        // Verify
        AreEqual(expected, path, "The path should be the more common path.");
    }

    /// <summary>
    /// Sets up the test environment when the page is loaded. The test instance is created. The session is cleared. The
    /// configurations are cleared.
    /// </summary>
    /// <param name="e">The event parameter.</param>
    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        Page.Session.Clear();
        ConfigManager.GetInstance().Clear();
        pathDiscovery = new DynamicPathDiscovery();
    }

    /// <summary>
    /// Cleans up the test environment when the page is unloaded. The test instance is disposed. The session is cleared.
    /// The configurations are cleared.
    /// </summary>
    /// <param name="e">The event parameter.</param>
    protected override void OnUnload(EventArgs e)
    {
        pathDiscovery = null;
        ConfigManager.GetInstance().Clear();
        Page.Session.Clear();

        base.OnUnload(e);
    }
</script>

<html xmlns="http://www.w3.org/1999/xhtml">
<head runat="server">
    <title>DynamicPathDiscoveryTest</title>
</head>
<body runat="server">
    <form id="form1" runat="server">
        <div>
            <asp:Button ID="GetPathBreadCrumbNodePageSiteMapInvalid" runat="server" OnClick="TestGetPathBreadCrumbNodePageSiteMapInvalid"
                Text="GetPathBreadCrumbNodePageSiteMapInvalid" />
            <asp:Button ID="GetPathBreadCrumbNodePageTwoMatchedAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageTwoMatchedAccuracy"
                Text="GetPathBreadCrumbNodePageTwoMatchedAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageMultipleRootsAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageMultipleRootsAccuracy"
                Text="GetPathBreadCrumbNodePageMultipleRootsAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageNoMatchAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageNoMatchAccuracy"
                Text="GetPathBreadCrumbNodePageNoMatchAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageLoopAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageLoopAccuracy"
                Text="GetPathBreadCrumbNodePageLoopAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageNormalAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageNormalAccuracy"
                Text="GetPathBreadCrumbNodePageNormalAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageGetSiteMapFail" runat="server" OnClick="TestGetPathBreadCrumbNodePageGetSiteMapFail"
                Text="GetPathBreadCrumbNodePageGetSiteMapFail" />
            <asp:Button ID="GetPathBreadCrumbNodePageISiteDiscoveryCreationFail" runat="server"
                OnClick="TestGetPathBreadCrumbNodePageISiteDiscoveryCreationFail" Text="GetPathBreadCrumbNodePageISiteDiscoveryCreationFail" />
            <asp:Button ID="GetPathBreadCrumbNodePageISiteDiscoverySourceFail" runat="server"
                OnClick="TestGetPathBreadCrumbNodePageISiteDiscoverySourceFail" Text="GetPathBreadCrumbNodePageISiteDiscoverySourceFail" />
            <asp:Button ID="GetPathBreadCrumbNodePageISiteDiscoveryWrongType" runat="server"
                OnClick="TestGetPathBreadCrumbNodePageISiteDiscoveryWrongType" Text="GetPathBreadCrumbNodePageISiteDiscoveryWrongType" />
            <asp:Button ID="GetPathBreadCrumbNodePageInvalidConfig" runat="server" OnClick="TestGetPathBreadCrumbNodePageInvalidConfig"
                Text="GetPathBreadCrumbNodePageInvalidConfig" />
            <asp:Button ID="GetPathBreadCrumbNodePageOnPreviousPathAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageOnPreviousPathAccuracy"
                Text="GetPathBreadCrumbNodePageOnPreviousPathAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageAppendPreviousPathAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageAppendPreviousPathAccuracy"
                Text="GetPathBreadCrumbNodePageAppendPreviousPathAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePagePreviousPathUnknownAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePagePreviousPathUnknownAccuracy"
                Text="GetPathBreadCrumbNodePagePreviousPathUnknownAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageShortestNoCommonAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageShortestNoCommonAccuracy"
                Text="GetPathBreadCrumbNodePageShortestNoCommonAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageShortestLessCommonAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageShortestLessCommonAccuracy"
                Text="GetPathBreadCrumbNodePageShortestLessCommonAccuracy" />
            <asp:Button ID="GetPathBreadCrumbNodePageMoreCommonAccuracy" runat="server" OnClick="TestGetPathBreadCrumbNodePageMoreCommonAccuracy"
                Text="GetPathBreadCrumbNodePageMoreCommonAccuracy" />
        </div>
    </form>
</body>
</html>
